/**
 * Introduction
 *
 * @title ROV - Easy applications from C
 *
 * @copyright Jason Tribbeck / 7th ARM 1994-2001
 *
 * @authors Jason Tribbeck
 *
 * @file list.h
 *
 * @type helper
 *
 * @description
 *
 * The list helper maintains a tree-based list of items within a canvas. It
 * can be used to display a tree of a filing system, or a threaded news
 * service viewer.
 * 
 * A list tree consists of one tree, which has a number of branches attached.
 * Each node on a branch corresponds to an individual item in the tree. Nodes
 * can have other branches attached to create a heirarchical arrangement.
 *
 * To display a node, each branch has a 'print' function whose function is to
 * render the node in a suitable format. Each node can have a handle, which 
 * the print function can use to display the node's information.
 *
 * By default, all nodes are text and sprite pairs, and a default printer is
 * available to display these.
 * 
 * Maintenance of the list (opening and closing of branches) is performed
 * automatically by the list helper.
 *
 * @note This is one of the most complex helpers in the ROVLib library, and was
 * only really used by one program. As this was written some time ago, this
 * documentation may be incorrect in places.
 *
 * @note Since documenting, the naming of the structures is probably incorrect.
 * Parameters to functions have been altered in this header file, but the name
 * of the definitions remains. This will probably change.
 */

#ifndef __rov_list
#define __rov_list


#ifdef __cplusplus
extern "C"
{
#endif

/**
 * Definition of a tree list
 */
typedef void *list_tree;

/**
 * Definition of a branch printer
 * @param branch the branch to display.
 * @param i the index into the branch.
 * @param x the x offset.
 * @param y the y offset.
 * @param h the handle to the node's data.
 * @param wimpflags standard WIMP flags for displaying the node (selected, faded).
 * @param fore the foreground colour.
 * @param back the background colour.
 * @see draw_printf.h
 * @see Introduction
 */
typedef void (* list_print)(list_tree branch,
			    int i,
			    int x,
			    int y,
			    void *h,
			    int wimpflags,
			    int fore,
			    int back);

/**
 * Definition of the head list.
 */
typedef void *list_main;

/**
 * The found list when clicking on a list.
 */
extern list_tree list_foundlist;

/**
 * The found item when clicking on a list.
 */
extern int list_founditem;

/**
 * Create a list branch.
 * @param items the number of items in the list.
 * @param flags the flags for this list (unused)
 */
list_tree list_create(int items,
		      int flags);

/**
 * Remove a list branch.
 * @param branch the branch to remove.
 * @see list_clear()
 */
void list_remove(list_tree branch);

/**
 * Clear a branch list
 * @param branch the branch to clear.
 * @note This does not remove the memory for this branch, rather
 * setting the number of added items to zero.
 * @see list_remove()
 */
void list_clear(list_tree branch);

/**
 * Attach a branch to a node in another branch.
 * @param parent the parent branch.
 * @param index the node index in the parent.
 * @param child the child branch.
 */
void list_attachlist(list_tree parent,
		     int index,
		     list_tree child);

/**
 * Add a node to a branch.
 * @param branch the branch.
 * @param text the pointer to the text.
 * @param spr_area the pointer to the sprite area.
 * @param spr_name the pointer to the sprite name.
 * @param handle the handle for the node.
 */
void list_add(list_tree branch,
	      char *text,
	      void *spr_area,
	      char *spr_name,
	      void *handle);

/**
 * Open/close a node.
 * @param branch the branch with the node to open.
 * @param index the node to open.
 * @param open non-zero to open, zero for close.
 */
void list_open(list_tree branch,
	       int index,
	       int open);

/**
 * Find out if a node is open or not.
 * @param branch the branch.
 * @param index the node to check.
 * @return non-zero for open node.
 */
int list_opened(list_tree branch,
		int index);

/**
 * Select/deselect a node.
 * @param branch the branch with the node to open.
 * @param index the node to open.
 * @param select non-zero to select, zero to deselect.
 */
void list_select(list_tree branch,
		 int index,
		 int select);

/**
 * Find out if a node is selected.
 * @param index the node to check.
 * @return non-zero for selected node.
 */
int list_selected(list_tree branch,
		  int index);

/**
 * Fade a node.
 * @param branch the branch with the node to open.
 * @param index the node to open.
 * @param fade non-zero to fade, zero to unfade
 */
int list_fade(list_tree branch,
	      int index,
	      int fade);

/**
 * Find out if a node is faded.
 * @param index the node to check.
 * @return non-zero for faded node.
 */
void list_faded(list_tree branch,
		int index);

/**
 * Find a selected node.
 * @param branch the branch to check.
 * @param index to start searching from (-1 to start).
 * @return the next selected node index, or -1 for no more.
 */
int list_findselected(list_tree branch,
		      int index);

/**
 * Count the selected items in a tree.
 * @param tree the tree to scan.
 * @return the number of selected nodes.
 */
int list_selecteditems(list_main tree);

/**
 * Redraw a node.
 * @param branch the branch containing the node.
 * @param index the index of the node.
 */
void list_redrawitem(list_tree branch,
		     int index);

/**
 * Change the sprite of a node.
 * @param branch the branch to change.
 * @param index the index of the node.
 * @param spr_area the sprite area.
 * @param spr_name the sprite name.
 */
void list_changesprite(list_tree branch,
		       int index,
		       void *spr_area,
		       char *spr_name);

/**
 * Set a list print function.
 * @param branch the branch.
 * @param printer the list printer.
 * @see list_print
 */
void list_setprintproc(list_tree branch,
		       list_print printer);

/**
 * Set the handle for a node.
 * @param branch the branch.
 * @param index the node index.
 * @param handle the handle.
 */
void list_sethandle(list_tree branch,
		    int index,
		    void *handle);

/**
 * Get the height of the list.
 * @param tree the tree to get the height of.
 * @return the height of the tree in OS units.
 */
int list_height(list_main tree);

/**
 * Attach a list to a canvas.
 * @param branch the main branch.
 * @param c the canvas.
 * @param name the name of the tree.
 * @param spr_area the sprite area.
 * @param spr_name the name of the title sprite.
 * @param fore the foreground colour.
 * @param back the background colour.
 * @param grey the greyed out colour.
 * @param x the (canvas) left hand coordinate of the list.
 * @param y the (canvas) top coordinate of the list.
 * @return the created tree.
 */
  
list_main list_attachtocanvas(list_tree branch,
			      canvas c,
			      char *name,
			      void *spr_area,
			      char *spr_name,
			      int fore,
			      int back,
			      int grey,
			      int x,
			      int y);

/**
 * Change the attachment of a list
 * @param tree the tree to change.
 * @param branch the new parent branch.
 */
void list_reattach(list_main tree,
		   list_tree branch);

/**
 * Draw a portion of a list.
 * @param tree the tree to redraw.
 * @param x0 the (canvas) left hand coordinate.
 * @param y0 the (canvas) bottom coordinate,
 * @param x1 the (canvas) right hand coordinate.
 * @param y1 the (canvas) top coordinate.
 * @note this is done during the canvas redraw.
 */
void list_draw(list_main tree,
	       int x0,
	       int y0,
	       int x1,
	       int y1);

/**
 * Find the branch of an item.
 * @param tree the tree.
 * @param y the (canvas) y coordinate.
 * @return the branch.
 * @see list_finditem()
 * @see list_handle()
 */
list_tree list_findlist(list_main tree,
			int y);

/**
 * Find the node in a branch.
 * @param branch the branch..
 * @param y the (canvas) y coordinate.
 * @return the node index.
 * @note Normally called after list_findlist().
 * @see list_findlist()
 * @see list_handle()
 */
int list_finditem(list_tree branch,
		  int y);

/**
 * Get the handle of an item.
 * @param branch the branch.
 * @param index the node index.
 * @return the handle.
 * @note Normally called after list_finditem().
 * @see list_findlist()
 * @see list_finditem()
 */
void *list_handle(list_tree branch,
		  int index);

/**
 * Handle a canvas click event.
 * @param tree the tree.
 * @param mouse_str the mouse event.
 * @return how the list handler handled the event.
 * @note Normally called in a canvas click event handler.
 * @see List click handler event codes
 * @see canvas.h
 */
int list_handleclick(list_main tree,
		     canvas_mousestr *mouse_str);

/**
 * Return the number of items in a branch.
 * @param branch the branch.
 * @return the number of nodes.
 */
int list_items(list_tree branch);

/**
 * Get the y coordinate of a node.
 * @param branch the branch.
 * @param index the node index.
 * @return the bottom y coordinate offset from the top of the list.
 */
int list_ycoordinate(list_tree branch,
		     int index);

/**
 * Deselect all items in a branch.
 * @param branch the branch to deselect.
 */
void list_deselectall(list_tree branch);

#define LIST_NOITEM (-1)

/**
 * @section List click handler event codes
 */
#define LIST_NOTHANDLED (-1)
#define LIST_HANDLED (0)
#define LIST_OPEN (1)
#define LIST_CLOSE (2)
#define LIST_LOAD (3)
#define LIST_SELECTED (4)
#define LIST_UNSELECTED (5)
#define LIST_FORCEOPEN (6)


#ifdef __cplusplus
}
#endif

#endif
